<?php

namespace App\Http\Controllers;

use App\Models\BMIRange;
use App\Models\BMIRangeExercise;
use Illuminate\Http\Request;
use App\Models\FitnessCategory;
use App\Models\ProgramExercise;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;

class ProgramExerciseController extends Controller
{

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $bmiRanges = BMIRange::all();
        $program_exercises = ProgramExercise::all();
        $fitnessCategories = FitnessCategory::all();
        return view('admin.program_exercise.index', compact('fitnessCategories', 'program_exercises', 'bmiRanges'));
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'fitness_category_id' => 'required|integer',
            'bmi_range_id' => 'required|array',
            'bmi_range_id.*' => 'integer|exists:bmi_ranges,id',
            'gender' => 'required',
            'exercise_type' => 'required|string',
            'exercise_name' => 'required|string',
            'sets' => 'required|integer',
            'reps' => 'required|integer',
            'rest' => 'required|integer',
            'description' => 'required|string',
        ]);

        if ($validator->fails()) {
            return redirect()->route('program-exercise.index')
                ->withErrors($validator)
                ->withInput();
        }

        // Process and store focus points (images)
        $focusPointUrls = [];

        if ($request->hasFile('focus_points')) {
            foreach ($request->file('focus_points') as $focusPointFile) {
                $extension = $focusPointFile->getClientOriginalExtension();
                $filename = time() . '_' . $focusPointFile->getClientOriginalName();
                $focusPointFile->move('uploads/focus_points/', $filename);
                $focusPointUrls[] = 'uploads/focus_points/' . $filename;
            }
        }

        // Process and store animation file
        $animationUrl = null;
        if ($request->hasFile('animation')) {
            $animationFile = $request->file('animation');
            $animationExtension = $animationFile->getClientOriginalExtension();
            $animationFilename = time() . '_animation.' . $animationExtension;
            $animationFile->move('uploads/animations/', $animationFilename);
            $animationUrl = 'uploads/animations/' . $animationFilename;
        }

        // Create the ProgramExercise instance
        $program_exercise = ProgramExercise::create(array_merge(
            $request->except('bmi_range_id', 'focus_points', 'animation'), // Exclude relevant fields
            ['focus_points' => implode(',', $focusPointUrls), 'animation' => $animationUrl]
        ));

        // Attach BMI range exercises
        foreach ($request->input('bmi_range_id') as $bmiRangeId) {
            $bmiRangeExercise = new BMIRangeExercise([
                'bmi_range_id' => $bmiRangeId,
                'program_exercise_id' => $program_exercise->id,
            ]);
            $bmiRangeExercise->save();
        }

        return redirect()->route('program-exercise.index')->with('success', 'Program Exercise created successfully');
    }


    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        $program_exercise = ProgramExercise::find($id);
        $fitnessCategories = FitnessCategory::all();
        $bmi_ranges = BMIRange::all();
        return view('admin.program_exercise.edit', compact('fitnessCategories', 'program_exercise', 'bmi_ranges'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, $id)
    {
        $validator = Validator::make($request->all(), [
            'fitness_category_id' => 'required|integer',
            'bmi_range_id' => 'required|array',
            'bmi_range_id.*' => 'integer|exists:bmi_ranges,id',
            'gender' => 'required',
            'exercise_type' => 'required|string',
            'exercise_name' => 'required|string',
            'sets' => 'required|integer',
            'reps' => 'required|integer',
            'rest' => 'required|integer',
            'description' => 'required|string',
        ]);

        if ($validator->fails()) {
            return redirect()->route('program-exercise.edit', $id)
                ->withErrors($validator)
                ->withInput();
        }
    
        $program_exercise = ProgramExercise::findOrFail($id);
    
        // Update the program exercise attributes
        $program_exercise->update($request->except('bmi_range_id'));
    
        // Delete existing BMIRangeExercise records for this program exercise
        $program_exercise->bmi_range_exercises()->delete();
    
        // Create new BMIRangeExercise records based on the updated request data
        foreach ($request->input('bmi_range_id') as $bmirangeId) {
            $bmirangeexercise = new BMIRangeExercise([
                'bmi_range_id' => $bmirangeId,
                'program_exercise_id' => $program_exercise->id,
            ]);
            $bmirangeexercise->save();
        }
    
        // Process and update focus points (images)
        $focusPointUrls = [];
    
        if ($request->hasFile('focus_points')) {
            foreach ($request->file('focus_points') as $focusPointFile) {
                $extension = $focusPointFile->getClientOriginalExtension();
                $filename = time() . '_' . $focusPointFile->getClientOriginalName();
                $focusPointFile->move('uploads/focus_points/', $filename);
                $focusPointUrls[] = 'uploads/focus_points/' . $filename;
            }
    
            // Update focus points as comma-separated string
            $program_exercise->focus_points = implode(',', $focusPointUrls);
            $program_exercise->save();
        }
    
        // Process and update animation file
        $animationUrl = null;
        if ($request->hasFile('animation')) {
            $animationFile = $request->file('animation');
            $animationExtension = $animationFile->getClientOriginalExtension();
            $animationFilename = time() . '_animation.' . $animationExtension;
            $animationFile->move('uploads/animations/', $animationFilename);
            $animationUrl = 'uploads/animations/' . $animationFilename;
    
            // Update animation URL
            $program_exercise->animation = $animationUrl;
            $program_exercise->save();
        }
    
        return redirect()->route('program-exercise.index')->with('success', 'Program Exercise updated successfully');
    }



    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        $program_exercise = ProgramExercise::find($id);
        $program_exercise->delete();
        return redirect()->route('program-exercise.index')->with('success', 'Program Exercise deleted successfully');
    }
}
